/***************************************************************************
 **
 ** Licensed Materials - Property of IBM
 **
 ** (C) COPYRIGHT International Business Machines Corp. 2000, 2001
 ** All Rights Reserved.
 **
 ** DB2 Everyplace Demo Engine                                     
 **                                                                         
 ** C++ interface to the DB2 Everyplace database.                           
 **                                                                         
 ***************************************************************************
 **                              DB2eEng.cpp                                
 ***************************************************************************/



/*****************************************************************************/
/**  Include files                                                          **/
/*****************************************************************************/

#include "DB2eEng.h"

/*****************************************************************************/
/*****************************************************************************/
/**  CVNurseEngine Class                                                    **/
/*****************************************************************************/
/*****************************************************************************/

/** CDB2eEngine Startup
  *   this methode starts the DB2 Everyplace engine and sets up the
  *   connection.
  *
  * @return FAILURE in any error case or SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eStartEngine()
{
  SQLTCHAR* server = _TEXT("C:/System/Data/DB2e/");
  SQLTCHAR* uid    = _TEXT("DB2Everyplace");
  SQLTCHAR* pwd    = _TEXT("DB2Everyplace");

  SQLHENV   henv;  
  SQLHDBC   hdbc;
  SQLHSTMT  hstmt;
  SQLRETURN rc;

  // try to allocate the environment handle
  rc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
  if (rc)
  { // if allocation fails
    iErrorLocation = _L("SQLAllocHandle:");
    iErrorMessage  = _L("Allocation of the environment handle failed.");
    iErrorState    = _L("");
    return FAILURE;
  }
  else
    this->setHEnv(henv);

  // try to allocate the database connection handle
  rc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
  if (SQL_SUCCESS != rc)
  { // if allocation fails
    SQLFreeHandle(SQL_HANDLE_ENV, henv);
    iErrorLocation = _L("SQLAllocHandle:");
    iErrorMessage  = _L("Allocation of the database connection handle failed.");
    iErrorState    = _L("");
    return FAILURE;
  }
  else
    this->setHDbc(hdbc);


  // start the connection to the database
  rc = SQLConnect(hdbc, server, SQL_NTS, uid, SQL_NTS, pwd, SQL_NTS);
  if (SQL_SUCCESS != rc)
  { // if connecting fails
    SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SQLFreeHandle(SQL_HANDLE_ENV, henv); 
    iErrorLocation = _L("SQLConnect:");
    iErrorMessage  = _L("Opening the database connection failed.");
    iErrorState    = _L("");
    return FAILURE;
  }

  // try to allocate the statement handle
  rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (SQL_SUCCESS != rc)
  { // if allocation fails
    SQLDisconnect(hdbc);
    SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
    SQLFreeHandle(SQL_HANDLE_ENV, henv); 
    iErrorLocation = _L("SQLAllocHandle:");
    iErrorMessage  = _L("Allocation of the statement handle failed.");
    iErrorState    = _L("");
    return FAILURE;
  }  
  else
    this->setHStmt(hstmt);

  return SUCCESS;
};




/** CDB2eEngine Stop
  *   this method stops the DB2 Everyplace engine and cleans up
  *   all environment-, connection- and statement handles.
  */

void CDB2eEngine::DB2eStopEngine()
{
  SQLFreeHandle(SQL_HANDLE_STMT, this->getHStmt());
  SQLDisconnect(this->getHDbc());
  SQLFreeHandle(SQL_HANDLE_DBC, this->getHDbc());

  SQLFreeHandle(SQL_HANDLE_ENV, this->getHEnv()); 
    
  // free memeory for the C standard library

  CloseSTDLIB();
};




/** CDB2eEngine execute SQL statement
  *   this method executes an SQL statement WITHOUT binding
  *   any parameters.
  *
  * @param sqlstmt: string of the SQL statement
  * @return FAILURE in any error case or SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eExecStatement(CHARTYPE* sqlstmt)
{
  SQLRETURN  rc;
  SQLHSTMT  hstmt = this->getHStmt();

  // prepare the SQL statement
  rc = SQLPrepare(hstmt, (SQLTCHAR*)sqlstmt, SQL_NTS);
  if (SQL_SUCCESS != rc)
    {
      DB2eErrorReport(hstmt, 
                      _L("DB2eExecStatement:"),
                      _L("Preparing SQL statement failed."));
      return FAILURE;
    }
  else
    {
      // execute the prepared SQL statement
      rc = SQLExecute(hstmt);
      if (SQL_SUCCESS != rc)
        {
          DB2eErrorReport(hstmt, 
                          _L("DB2eExecStatement:"),
                          _L("Executing SQL statement failed."));
          return FAILURE;
        }
    }

  return SUCCESS;
};




/** CDB2eEngine execute binded SQL statement
  *   this method executes an SQL statement WITH binding
  *   a parameter list.
  *
  * @param sqlstmt: string of the SQL statement
  * @param param: list of parameters
  * @return FAILURE in any error case or SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eExecBindStatement(CHARTYPE* sqlstmt, DB2eBindParam* param)
{
  SQLRETURN        rc;
  SQLHSTMT        hstmt = this->getHStmt();
  DB2eBindParam*  p      = param;

  // prepare the SQL statement
  rc = SQLPrepare(hstmt, (SQLTCHAR*)sqlstmt, SQL_NTS);
  if (SQL_SUCCESS != rc)
    {
      DB2eErrorReport(hstmt, 
                      _L("DB2eExecBindStatement:"),
                      _L("Preparing SQL statement failed."));
      return FAILURE;
    }
  else
    {
      // bind all parameters of the list
      while (p != NULL)
      {
        // bind the n-th parameter of the list
        rc = SQLBindParameter(hstmt, 
                              (SQLUSMALLINT)  p->number,
                              SQL_PARAM_INPUT,
                              (SQLSMALLINT)   p->ctype,
                              (SQLSMALLINT)   p->type,
                              (SQLUINTEGER)   p->columnSize,
                              (SQLSMALLINT)   p->decimalDigits,
                              (SQLPOINTER)    p->pValue,
                              (SQLINTEGER)    p->buffLength,
                              (SQLINTEGER*)   p->length);
        if (SQL_SUCCESS != rc)
          {
            DB2eErrorReport(hstmt, 
                            _L("DB2eExecBindStatement:"),
                            _L("Binding SQL statement failed."));
            return FAILURE;
          };

        p = p->next; // step in the list one parameter forward
      }

      // execute binded SQL statement
      rc = SQLExecute(hstmt);
      if (SQL_SUCCESS != rc)
        {
          DB2eErrorReport(hstmt, 
                          _L("DB2eExecBindStatement:"),
                          _L("Executing SQL statement failed."));
          return FAILURE;
        }
    }

  return SUCCESS;
};





/** CDB2eEngine fetch data 
  *   this method fetches the next data record of the 
  *   last executed SQL statement.
  *
  * @return FAILURE in any error case 
  * @return NODATA if no recod available 
  * @return SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eFetchData()
{
  SQLRETURN  rc    = 0;
  SQLHSTMT  hstmt = this->getHStmt();

  // fetch the next record
  rc = SQLFetch(hstmt);
  if (SQL_SUCCESS != rc)
    {
      if (SQL_NO_DATA_FOUND != rc)
      {
        DB2eErrorReport(hstmt, 
                        _L("DB2eFetchData:"),
                        _L("Fetching data failed."));
        return FAILURE;
      }
      else
        return NODATA;
    }
  else
    return SUCCESS;
};






/** CDB2eEngine get CHAR, DECIMAL, DATE, TIME, VARCHAR data 
  *   this method get the data of a certain column of the  
  *   last fetched record.
  *
  * @param  colNum = column number to get the data
  * @param  val    = pointer to a character buffer
  * @param  size   = size of the buffer
  * @param  length = length of the value stored in val
  * @return FAILURE in any error case 
  * @return SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eGetData(int       colNum, 
                                    CHARTYPE* val, 
                                    long      size, 
                                    long*     length)
{
  SQLRETURN  rc    = 0;
  SQLHSTMT  hstmt = this->getHStmt();

  // get data 
  rc = SQLGetData(hstmt, 
                  (unsigned short) colNum, 
                  SQL_C_CHAR, 
                  (SQLTCHAR*)val, 
                  size, 
                  (SQLINTEGER *) length);

  if (SQL_SUCCESS != rc)
  {
    DB2eErrorReport(hstmt, 
                    _L("DB2eGetData:"),
                    _L("Getting data failed. (char)"));
    return FAILURE;
  }

  return SUCCESS;
};





/** CDB2eEngine get SMALLINT data 
  *   this method get the data of a certain column of the  
  *   last fetched record.
  *
  * @param  colNum = column number to get the data
  * @param  val    = pointer to a short
  * @param  length = length of the value stored in val
  * @return FAILURE in any error case 
  * @return SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eGetData(int colNum, short* val, long* length)
{
  SQLRETURN  rc = 0;
  SQLHSTMT  hstmt = this->getHStmt();

  // get data
  rc = SQLGetData(hstmt, 
                  (unsigned short) colNum, 
                  SQL_C_SHORT, 
                  val, 
                  sizeof(short), 
                  (SQLINTEGER *) length);

  if (SQL_SUCCESS != rc)
  {
    DB2eErrorReport(hstmt, 
                    _L("DB2eGetData:"),
                    _L("Getting data failed. (short)"));
    return FAILURE;
  }

  return SUCCESS;
};





/** CDB2eEngine get INTEGER data 
  *   this method get the data of a certain column of the  
  *   last fetched record.
  *
  * @param  colNum = column number to get the data
  * @param  val    = pointer to a long
  * @param  length = length of the value stored in val
  * @return FAILURE in any error case 
  * @return SUCCESS otherwise
  */

DB2eRETURN CDB2eEngine::DB2eGetData(int colNum, long* val, long* length)
{
  SQLRETURN  rc = 0;
  SQLHSTMT  hstmt = this->getHStmt();

  // get data
  rc = SQLGetData(hstmt, 
                  (unsigned short) colNum, 
                  SQL_C_LONG, 
                  val, 
                  sizeof(long), 
                  (SQLINTEGER *) length);

  if (SQL_SUCCESS != rc)
  {
    DB2eErrorReport(hstmt, 
                    _L("DB2eGetData:"),
                    _L("Getting data failed. (long)"));
    return FAILURE;
  }

  return SUCCESS;
};



/** CDB2eEngine get the error state
  *   this method returns either the SQL error state or the 
  *   return code of SQLGetDiagRec(). 
  *
  * @return string of the error message
  */

TPtrC CDB2eEngine::DB2eGetErrorState()
{
  return (this->iErrorState);
};


/** CDB2eEngine get the error message
  *   this method returns the error message string 
  *
  * @return string of the error message
  */

TPtrC CDB2eEngine::DB2eGetErrorMsg()
{
  return (this->iErrorMessage);
};




/** CDB2eEngine get the error message
  *   this method returns the error message string 
  *
  * @return string of the error message
  */

TPtrC CDB2eEngine::DB2eGetErrorLocation()
{
  return (this->iErrorLocation);
};





/** CDB2eEngine get the error message from the DB2 Everyplace engine
  *   this method creates the error message string 
  *
  * @param hstmt = statement handle of the executed SQL statement
  * @param place = description of the place where the error appears
  */

void CDB2eEngine::DB2eErrorReport(SQLHSTMT hstmt, 
                                  TPtrC loc,
                                  TPtrC msg)
{
  SQLRETURN  rc;
  SQLTCHAR sqlstate[8], errmsg[80];
  long nativeErr;
  short  msglen;

  // get the error message from the DB2 Everyplace engine
  rc = SQLGetDiagRec(SQL_HANDLE_STMT, hstmt, 1, sqlstate, 
                    &nativeErr, errmsg, 80, &msglen);

  // copy the place and the error code
  
  TBuf<DB2eERROR_LOCATION_LENGTH> locbuf(loc);
  iErrorLocation.Copy(locbuf);

  TBuf<DB2eERROR_MESSAGE_LENGTH> msgbuf(msg);
  iErrorMessage.Copy(msgbuf);
  
  TBuf<DB2eERROR_STATE_LENGTH> statebuf;
  if (rc != 0)
  {
    statebuf.Copy(_L("["));
    statebuf.AppendNum(rc);
    statebuf.Append(_L("]"));
  }
  else
  {
    statebuf.Copy(_L("<"));
    statebuf.Append(TPtrC(sqlstate));
    statebuf.Append(_L(">"));
    statebuf.SetLength(7);
  }
  iErrorState.Copy(statebuf);

  return;
};






/** CDB2eEngine get internal data
  *   this returns the environment handle
  *
  * @return environment handle
  */

SQLHENV CDB2eEngine::getHEnv()
{
  return iHenv;
};

/** CDB2eEngine get internal data
  *   this returns the database connection handle
  *
  * @return database connection handle
  */

SQLHDBC CDB2eEngine::getHDbc()
{
  return iHdbc;
};

/** CDB2eEngine get internal data
  *   this returns the statement handle
  *
  * @return statement handle
  */

SQLHSTMT CDB2eEngine::getHStmt()
{
  return iHstmt;
};





/** CDB2eEngine set internal data
  *   this sets the environment handle
  *
  * @param henv = environment handle
  */

void CDB2eEngine::setHEnv(SQLHENV henv)
{
  iHenv = henv;
};


/** CDB2eEngine set internal data
  *   this sets the database connection handle
  *
  * @param hdbc = database connection handle
  */

void CDB2eEngine::setHDbc(SQLHDBC hdbc)
{
  iHdbc = hdbc;
};


/** CDB2eEngine set internal data
  *   this sets the statement handle
  *
  * @param hstmt = statement handle
  */

void CDB2eEngine::setHStmt(SQLHSTMT hstmt)
{
  iHstmt = hstmt;
};


